import os.path

# interactive commands

import command

class BufferWriteAssociated(command.Command):
	"""Save a buffer to the file it is associated with. This is a low-level
	command, used by buffer-write and buffer-save.
	"""
	name = 'buffer-write-associated'
	context = 'buffer'
	def call(self, buf):
		filpath = os.path.expanduser(buf.filename)
		(dir, fil) = os.path.split(filpath)
		ix = 0
		ixst = ''
		while 1:
			newfilpath = os.path.join(dir, '#'+fil+'#'+ixst)
			if (not os.path.exists(newfilpath)):
				break
			ix = ix+1
			ixst = str(ix)
		fl = open(newfilpath, 'w')
		try:
			buf.serialize(fl)
		finally:
			fl.close()
		os.rename(newfilpath, filpath)
		buf.isdirty = 0
		buf.watch.signal(watcher.SIG_MODE)
		status.post('Wrote "'+buf.filename+'".')
		
class BufferWrite(command.Command):
	"""Save a buffer to a file. The buffer will be associated with this
	file for future saves. The buffer is renamed to reflect this.
	"""
	name = 'buffer-write'
	context = 'buffer'
	def call(self, buf):
		bufpath = inputstring.input_filename_arg(self, 1, 'File to save in', self.name)
		buf.checkvalid()
		filpath = os.path.expanduser(bufpath)
		if (os.path.exists(filpath)):
			res = inputkey.input_yesorno(self, 'Overwrite existing file "'+bufpath+'"?')
			if (not res):
				raise command.StatusWarning('Cancelled.')
		buffer.rename(buf, os.path.basename(filpath))
		buf.filename = bufpath
		buf.watch.signal(watcher.SIG_MODE)
		buf.checkvalid()
		self.runcmd('buffer-write-associated', buf)

class BufferSave(command.Command):
	"""Save a buffer to the file it's associated with. If no changes
	need to be saved, do nothing. If no file is associated with the buffer,
	choose one.
	"""
	name = 'buffer-save'
	context = 'buffer'
	def call(self, buf):
		if (not buf.isdirty):
			raise command.StatusWarning('(No changes need to be saved.)')
		if (buf.filename == None):
			return self.runcmd('buffer-write', buf)
		buf.checkvalid()
		self.runcmd('buffer-write-associated', buf)

command.register([
	BufferSave, BufferWrite, BufferWriteAssociated
])
		
command.bind('buffer-save', '^X ^S')
command.bind('buffer-write', '^X ^W')

# late imports
import log
import status
import buffer
import watcher
import inputkey
import inputstring
