import string
import command

helpoptions = {
	'l' : ('help-license', 'Display Umics\'s license for copying and distribution'),
	'c' : ('describe-command', 'Describe an interactive command'),
	'k' : ('describe-key', 'Describe a key or key sequence'),
	'?' : ('help-options', 'Display the help menu options'),
}

class HelpMenu(command.Command):
	"""The Umics help menu.
	"""
	name = 'help-menu'
	def call(self, con):
		try:
			while 1:
				(cmdname, label) = inputkey.input_choice_arg(
					self, 
					1, 
					'Help (type ? for options): ',
					helpoptions)
				self.runcmdfree(cmdname)
				if (cmdname != 'help-options'):
					break
		finally:
			buf = buffer.findbyname('*help-options*')
			if (buf):
				buf.ensureclosed()
			

class HelpLicense(command.Command):
	"""Display Umics's license for copying and distribution.
	This is the GNU GPL (General Public License).
	"""
	name = 'help-license'
	def call(self, con):
		log.clear('LICENSE')
		log.log(literaltext.License, 'LICENSE')

class HelpDescribeCommand(command.Command):
	"""Describe a command, given its name.
	"""	
	name = 'describe-command'
	def call(self, con):
		complet = command.getcompletor()
		cmdname = inputstring.input_string_arg(
			self, 
			1,
			'Describe command', 
			self.name,
			complet)
		res = command.commands[cmdname]
		logo = log.Logger('*help*', 1)
		logo.write(cmdname+':\n\n')
		logo.write('    ('+str(res)+' in module '+str(res.__module__)+')\n')
		logo.write('    (available in context '+str(res.context)+')\n\n')
		if (res.__doc__):
			logo.write(res.__doc__.replace('\t','    '))
		
class HelpDescribeKey(command.Command):
	"""Describe a command, given the key sequence that invokes it.
	"""	
	name = 'describe-key'
	def call(self, con):
		status.post('Describe key:')
		(res, con, resargs, keylist) = inputkey.input_cmdsequence(self)
		cmdname = res.name
		logo = log.Logger('*help*', 1)
		logo.write(string.join(map(command.keyname, keylist))
			+' runs the command '+cmdname)
		if (len(resargs)):
			logo.write(' with argument ')
			if (len(resargs) == 1):
				logo.write('('+str(resargs[0])+')')
			else:
				logo.write(str(resargs))
		logo.write('\n\n')
		logo.write(cmdname+':\n\n')
		logo.write('    ('+str(res)+' in module '+str(res.__module__)+')\n')
		logo.write('    (available in context '+str(res.context)+')\n\n')
		if (res.__doc__):
			logo.write(res.__doc__.replace('\t','    '))

			
class HelpOptions(command.Command):
	name = 'help-options'
	def call(self, con):
		logo = log.Logger('*help-options*', 1)
		ls = helpoptions.keys()
		ls.sort()
		logo.write('Type a help option:\n\n')
		for key in ls:
			(cmdname, label) = helpoptions[key]
			logo.write(key+' ('+cmdname+'): '+label+'\n')

command.register([
	HelpMenu, HelpOptions,
	HelpDescribeCommand, HelpDescribeKey,
	HelpLicense
])

command.bind('help-menu', '?')

import log
import status
import buffer
import execute
import inputkey
import inputstring
import literaltext

### need a way to stick a textbuffer at its top

