import command

class SelwinDown(command.Command):
	"""Move the selection down one line.
	"""
	name = 'selwin-down'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = min(win.selection+win.numcols, len(win.layout)-1)
		win.dirtyfocus = 1

class SelwinRight(command.Command):
	"""Move the selection right one column.
	"""
	name = 'selwin-right'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = min(win.selection+1, len(win.layout)-1)
		win.dirtyfocus = 1

class SelwinUp(command.Command):
	"""Move the selection up one line.
	"""
	name = 'selwin-up'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = max(win.selection-win.numcols, 0)
		win.dirtyfocus = 1

class SelwinLeft(command.Command):
	"""Move the selection left one column.
	"""
	name = 'selwin-left'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = max(win.selection-1, 0)
		win.dirtyfocus = 1

### page up and page down code doesn't really jump the right distance
		
class SelwinPageDown(command.Command):
	"""Move the selection down one page.
	"""
	name = 'selwin-page-down'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = min(win.selection+win.numcols*(win.height-1),
			len(win.layout)-1)
		win.dirtyfocus = 1

class SelwinPageUp(command.Command):
	"""Move the selection up one page.
	"""
	name = 'selwin-page-up'
	context = 'selwindow'
	def call(self, win):
		if (len(win.layout)==0):
			return
		win.selection = max(win.selection-win.numcols*(win.height-1),
			0)
		win.dirtyfocus = 1

class SelwinAccept(command.Command):
	"""Accept the selection.
	"""
	name = 'selwin-accept'
	context = 'selwindow'
	def call(self, win):
		if (win.selection < 0 or win.selection >= len(win.layout)):
			raise command.StatusBeepWarning('No selection')
		if (not minibuf.active):
			raise command.StatusBeepWarning('Minibuffer not active')
		con = minibuf.buf
		st = win.data.body[win.selection]
		con.replace(st, 0, len(con.body))
		con.setmark(len(con.body))
		raise command.DoneException

class SelwinEnter(command.Command):
	"""Enter the selection into the minibuffer.
	"""
	name = 'selwin-enter'
	context = 'selwindow'
	def call(self, win):
		if (win.selection < 0 or win.selection >= len(win.layout)):
			raise command.StatusBeepWarning('No selection')
		if (not minibuf.active):
			raise command.StatusBeepWarning('Minibuffer not active')
		con = minibuf.buf
		st = win.data.body[win.selection]
		con.replace(st, 0, len(con.body))
		con.setmark(len(con.body))
		window.changefocus(-1)

command.register([
	SelwinDown, SelwinUp,
	SelwinRight, SelwinLeft,
	SelwinPageDown, SelwinPageUp,
	SelwinEnter, SelwinAccept
])

command.bind('selwin-down', 'down')
command.bind('selwin-down', '^N')
command.bind('selwin-up', 'up')
command.bind('selwin-up', '^P')
command.bind('selwin-page-down', 'pagedown')
command.bind('selwin-page-down', '^V')
command.bind('selwin-page-up', 'pageup')
command.bind('selwin-page-up', 'esc v')
command.bind('selwin-right', 'right')
command.bind('selwin-right', '^F')
command.bind('selwin-left', 'left')
command.bind('selwin-left', '^B')
command.bind('selwin-accept', 'return')
command.bind('selwin-enter', 'tab')

# late imports
import minibuf
import window
